#qpy:kivy
#
# A High Intensity Interval Training app.
# Copyright (C) 2017 MiKael NAVARRO
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""A High Intensity Interval Training app.
Copyright (C) 2017 MiKael NAVARRO

A simple/portable High Intensity Interval Training app written with Kivy.

"""

# Specific variables for pydoc
__author__ = "MiKael Navarro <klnavarro@gmail.com>"
__date__ = "Sat Sep 09 2017"
__version__ = "0.1.2"
__credits__ = """Thanks to KeyWeeUsr for his example."""

# Include directives
import kivy
kivy.require('1.7.2')
 
from kivy.app import App
from kivy.core.window import Window
from kivy.uix.screenmanager import ScreenManager, Screen, SlideTransition
from kivy.properties import NumericProperty, StringProperty, ListProperty, ObjectProperty
from kivy.clock import Clock
from kivy.core.audio import SoundLoader
import datetime


# Global definitions
#Window.size = (360, 640)

#
# Utility funcs.
#
def timestamp_to_seconds(timestr):
    """Convert 'SS' or 'MM:SS' or 'HH:MM:SS' (although not necessarily two digits per part) to seconds."""

    pt = datetime.datetime.strptime(timestr, '%M:%S')
    #seconds = pt.second + pt.minute * 60
    #seconds = int(datetime.timedelta(minutes=int(pt.minute),seconds=int(pt.second)).total_seconds())
    seconds = 0
    for part in timestr.split(':'):
        seconds = seconds * 60 + int(part)
    return seconds

def secs_to_MS(secs):
    """Convert seconds to 'minutes:seconds' string format."""

    return datetime.datetime.fromtimestamp(secs).strftime('%M:%S')

def secs_to_HMS(secs):
    """Convert seconds to 'hour:minutes:seconds' string format."""

    if secs < 3600:
        return datetime.datetime.fromtimestamp(secs).strftime('%M:%S')
    else:
        return datetime.datetime.fromtimestamp(secs).strftime('%H:%M:%S')


#
# Setup screen.
#
class SetupScreen(Screen):
    """Setup Screen."""

    def enforce_numeric(self, text_input):
        """Ensure the text input contains only numeric characters or nothing."""

        if not text_input.text.isdigit():
            digit_list = [num for num in text_input.text if num.isdigit()]
            text_input.text = "".join(digit_list)

    def validate_duration(self, text_input):
        print "validate got ", str(text_input.text)

    def start(self, config):
        # Update config values
        config.set("workout", "preptime", str(timestamp_to_seconds(self.ids.preptime.text)))
        config.set("workout", "worktime", str(timestamp_to_seconds(self.ids.worktime.text)))
        config.set("workout", "resttime", str(timestamp_to_seconds(self.ids.resttime.text)))
        config.set("workout", "nbrounds", self.ids.nbrounds.text)
        config.set("workout", "cooldown", str(timestamp_to_seconds(self.ids.cooldown.text)))
        config.write()

        if __debug__:
            print "start():"
            print "preptime= %s" % secs_to_MS(config.getint("workout", "preptime"))
            print "worktime= %s" % secs_to_MS(config.getint("workout", "worktime"))
            print "resttime= %s" % secs_to_MS(config.getint("workout", "resttime"))
            print "nbrounds= %d" % config.getint("workout", "nbrounds")
            print "cooldown= %s" % secs_to_MS(config.getint("workout", "cooldown"))

        # Calculate total time
        tottime = config.getint("workout", "preptime") \
          + (config.getint("workout", "worktime") \
                + config.getint("workout", "resttime")) \
          * config.getint("workout", "nbrounds") \
          + config.getint("workout", "cooldown")
        self.manager.get_screen("workout").ids.totaltime.text = secs_to_MS(tottime)
        if __debug__:
            print "totaltime= %s (%ds)" % (secs_to_MS(tottime), tottime)

        # Now, prepare workout plan
        self.manager.get_screen("workout").hiit = []
        self.manager.get_screen("workout").hiit.append({'name': "Prep Time", 
                                                        'duration': config.getint("workout", "preptime"),
                                                        'color': (1,1,0,1), 'num': 0})
        nbrounds = config.getint("workout", "nbrounds")
        for n in range(nbrounds):
            self.manager.get_screen("workout").hiit.append({'name': "Work Time %d/%d" % (n+1,nbrounds),
                                                            'duration': config.getint("workout", "worktime"),
                                                            'color': (1,0,0,1), 'num': n+1})
            self.manager.get_screen("workout").hiit.append({'name': "Rest Time %d/%d" % (n+1,nbrounds),
                                                            'duration': config.getint("workout", "resttime"),
                                                            'color': (0,1,0,1), 'num': n+1})
        self.manager.get_screen("workout").hiit.append({'name': "Cool Down",
                                                        'duration': config.getint("workout", "cooldown"),
                                                        'color': (0,1,1,1), 'num': 0})
        self.manager.get_screen("workout").cycle = len(self.manager.get_screen("workout").hiit)
        if __debug__:
            print "hiit(%d)= %s" % (self.manager.get_screen("workout").cycle, self.manager.get_screen("workout").hiit)

        # Init main counter with prep time
        self.manager.get_screen("workout").ids.workouttitle.text = self.manager.get_screen("workout").hiit[0]['name']
        self.manager.get_screen("workout").ids.countdown.text = secs_to_MS(self.manager.get_screen("workout").hiit[0]['duration'])
        self.manager.get_screen("workout").ids.countdown.color = self.manager.get_screen("workout").hiit[0]['color']

        self.manager.transition.direction = 'left'
        self.manager.current = "workout"


#
# Workout screen.
#
class WorkoutScreen(Screen):
    """Workout screen."""

    hiit = ListProperty()
    cycle = NumericProperty()

    def count_down(self, dt):
        """Count down callback."""

        seconds = timestamp_to_seconds(self.ids.countdown.text) - 1
        tottime = timestamp_to_seconds(self.ids.totaltime.text) - 1  # decr total time too

        if seconds >= 0:
            self.ids.countdown.text = secs_to_MS(seconds)
            self.ids.totaltime.text = secs_to_MS(tottime)
            if seconds == 0: 
                beep.play()  # beep!
        else:
            if tottime <= 0 or self.cycle <= 0:
                self.stop()
                if __debug__: print "end."
            else:
                self.cycle = self.cycle - 1
                if __debug__: print "next cycle...",
                # Switch to next workout time
                self.ids.workouttitle.text = self.hiit[len(self.hiit) - self.cycle]['name']
                self.ids.countdown.text = secs_to_MS(self.hiit[len(self.hiit) - self.cycle]['duration'])
                self.ids.countdown.color = self.hiit[len(self.hiit) - self.cycle]['color']
        
    def start(self):
        """Start timer."""

        if __debug__:
            print "hiit(%d)= %s" % (self.cycle, self.hiit)

        self.ids.pausebtn.text = "Pause" 
        self.ids.pausebtn.state = 'down' 
        self.ids.countdown.state = 'down' 

        Clock.unschedule(self.count_down)  # NOT RECOMMENDED???
        Clock.schedule_interval(self.count_down, 1)

    def stop(self):
        """Stop timer."""

        self.ids.pausebtn.text = "Resume" 
        self.ids.pausebtn.state = 'normal' 
        self.ids.countdown.state = 'normal' 

        Clock.unschedule(self.count_down)  # NOT RECOMMENDED???

    def update(self, state):
        """Start/stop timer."""

        if state == 'down':
            self.start()
        elif state == 'normal':
            self.stop()


#
# Main screen.
#
class RootScreen(ScreenManager):
    """Main screen."""

    pass


#
# Main App class.
#
class HIITApp(App):
    """A High Intensity Interval Training app (implicitly try to load 'hiit.kv')."""

    icon = "kvhiit.png"  # prior to 1.8.0

    def build_config(self, config):
        """Default configuration file 'hiit.ini'."""

        config.setdefaults("workout", {
            "preptime": str(12 * 60),
            "worktime": str(30),
            "resttime": str(30),
            "nbrounds": str(6),
            "cooldown": str(10 * 60)
        })

    def build(self):
        # Initializes the application; it will be called only once.
        root = RootScreen() #WorkoutScreen()

        if __debug__:
            print "build():"
            print "preptime= %s" % secs_to_MS(self.config.getint("workout", "preptime"))
            print "worktime= %s" % secs_to_MS(self.config.getint("workout", "worktime"))
            print "resttime= %s" % secs_to_MS(self.config.getint("workout", "resttime"))
            print "nbrounds= %d" % self.config.getint("workout", "nbrounds")
            print "cooldown= %s" % secs_to_MS(self.config.getint("workout", "cooldown"))

        # Init setup screen
        root.ids.setup_screen.ids.preptime.text = secs_to_MS(self.config.getint("workout", "preptime"))
        root.ids.setup_screen.ids.worktime.text = secs_to_MS(self.config.getint("workout", "worktime"))
        root.ids.setup_screen.ids.resttime.text = secs_to_MS(self.config.getint("workout", "resttime"))
        root.ids.setup_screen.ids.nbrounds.text = self.config.get("workout", "nbrounds")
        root.ids.setup_screen.ids.cooldown.text = secs_to_MS(self.config.getint("workout", "cooldown"))

        # Calculate total time
        tottime = self.config.getint("workout", "preptime") \
          + (self.config.getint("workout", "worktime") \
                 + self.config.getint("workout", "resttime")) \
          * self.config.getint("workout", "nbrounds") \
          + self.config.getint("workout", "cooldown")
        root.ids.workout_screen.ids.totaltime.text = secs_to_MS(tottime)
        if __debug__:
            print "totaltime= %s (%ds)" % (secs_to_MS(tottime), tottime)

        return root

    use_kivy_settings = False
    def build_settings(self, settings):
        """Create a settings panel."""

        jsondata = """[{ "type": "title",
                         "title": "Workout session" },
                          { "title": "Prep Time",
                            "desc": "Prep time (in seconds).",
                            "section": "workout",
                            "key": "preptime",
                            "type": "numeric" },
                          { "title": "Work",
                            "desc": "Work time (in seconds).",
                            "section": "workout",
                            "key": "worktime",
                            "type": "numeric" },
                          { "title": "Rest Time",
                            "desc": "Rest time (in seconds).",
                            "section": "workout",
                            "key": "resttime",
                            "type": "numeric" },
                          { "title": "Rounds",
                            "desc": "Nb rounds work/rest time.",
                            "section": "workout",
                            "key": "nbrounds",
                            "type": "numeric" },
                          { "title": "Cool Down Time",
                            "desc": "Cool down time (in seconds).",
                            "section": "workout",
                            "key": "cooldown",
                            "type": "numeric" }
                      ]"""
        settings.add_json_panel("HIIT app", self.config, data=jsondata)

    def params(self):
        """Display settings panel (F1 or via a long press of the 'task/recent apps' button)."""

        self.open_settings()  # display settings panel

    def on_config_change(self, config, section, key, value):
        """Update UI."""

        if config is self.config:
            if __debug__:
                token = (section, key, value)
                print "on_config_change(section=%s, key=%s, value=%s):" % token
                print "preptime= %s" % secs_to_MS(config.getint("workout", "preptime"))
                print "worktime= %s" % secs_to_MS(config.getint("workout", "worktime"))
                print "resttime= %s" % secs_to_MS(config.getint("workout", "resttime"))
                print "nbrounds= %d" % config.getint("workout", "nbrounds")
                print "cooldown= %s" % secs_to_MS(config.getint("workout", "cooldown"))

            # Recalc total time
            tottime = config.getint("workout", "preptime") \
              + (config.getint("workout", "worktime") \
                     + config.getint("workout", "resttime")) \
              * config.getint("workout", "nbrounds") \
              + config.getint("workout", "cooldown")
            self.root.ids.workout_screen.ids.totaltime.text = secs_to_MS(tottime)
            if __debug__:
                print "totaltime= %s (%ds)" % (secs_to_MS(tottime), tottime)

            # Re-init main counter with prep time
            self.root.ids.workout_screen.ids.countdown.text = secs_to_MS(config.getint("workout", "preptime"))

            # Updt setup screen
            self.root.ids.setup_screen.ids.preptime.text = secs_to_MS(config.getint("workout", "preptime"))
            self.root.ids.setup_screen.ids.worktime.text = secs_to_MS(config.getint("workout", "worktime"))
            self.root.ids.setup_screen.ids.resttime.text = secs_to_MS(config.getint("workout", "resttime"))
            self.root.ids.setup_screen.ids.nbrounds.text = config.get("workout", "nbrounds")
            self.root.ids.setup_screen.ids.cooldown.text = secs_to_MS(config.getint("workout", "cooldown"))
                
    def on_pause(self):
        # Here we can save data if needed
        return True

    def on_resume(self):
        # Here we can check if any data needs replacing (usually nothing)
        pass


#
# Main entry point:
#
if __name__ == "__main__":

    # Prepare sound
    beep = SoundLoader.load('snd/beep.ogg')
    if __debug__ and beep:
        print("Sound found at %s" % beep.source)
        print("Sound is %.3f seconds" % beep.length)
        #beep.play()

    HIITApp().run()
